@@ -66,6 +66,7 @@ macro_rules! define_keywords {
 
 // The following keywords should be sorted to be able to match using binary search
 define_keywords!(
+    ABORT,
     ABS,
     ACTION,
     ADD,
@@ -103,6 +104,7 @@ define_keywords!(
     BOTH,
     BY,
     BYTEA,
+    CACHE,
     CALL,
     CALLED,
     CARDINALITY,
@@ -120,6 +122,7 @@ define_keywords!(
     CHECK,
     CLOB,
     CLOSE,
+    CLUSTER,
     COALESCE,
     COLLATE,
     COLLECT,
@@ -127,6 +130,7 @@ define_keywords!(
     COLUMNS,
     COMMIT,
     COMMITTED,
+    COMPUTE,
     CONDITION,
     CONNECT,
     CONSTRAINT,
@@ -157,6 +161,7 @@ define_keywords!(
     CURRENT_USER,
     CURSOR,
     CYCLE,
+    DATABASE,
     DATE,
     DAY,
     DEALLOCATE,
@@ -165,13 +170,16 @@ define_keywords!(
     DECLARE,
     DEFAULT,
     DELETE,
+    DELIMITED,
     DENSE_RANK,
     DEREF,
     DESC,
     DESCRIBE,
     DETERMINISTIC,
+    DIRECTORY,
     DISCONNECT,
     DISTINCT,
+    DISTRIBUTE,
     DOUBLE,
     DROP,
     DYNAMIC,
@@ -195,6 +203,7 @@ define_keywords!(
     EXTENDED,
     EXTERNAL,
     EXTRACT,
+    FAIL,
     FALSE,
     FETCH,
     FIELDS,
@@ -206,6 +215,7 @@ define_keywords!(
     FOLLOWING,
     FOR,
     FOREIGN,
+    FORMAT,
     FRAME_ROW,
     FREE,
     FROM,
@@ -220,15 +230,18 @@ define_keywords!(
     GROUPS,
     HAVING,
     HEADER,
+    HIVEVAR,
     HOLD,
     HOUR,
     IDENTITY,
     IF,
+    IGNORE,
     IN,
     INDEX,
     INDICATOR,
     INNER,
     INOUT,
+    INPUTFORMAT,
     INSENSITIVE,
     INSERT,
     INT,
@@ -262,11 +275,13 @@ define_keywords!(
     LOCALTIMESTAMP,
     LOCATION,
     LOWER,
+    MANAGEDLOCATION,
     MATCH,
     MATERIALIZED,
     MAX,
     MEMBER,
     MERGE,
+    METADATA,
     METHOD,
     MIN,
     MINUTE,
@@ -274,6 +289,7 @@ define_keywords!(
     MODIFIES,
     MODULE,
     MONTH,
+    MSCK,
     MULTISET,
     NATIONAL,
     NATURAL,
@@ -284,6 +300,7 @@ define_keywords!(
     NO,
     NONE,
     NORMALIZE,
+    NOSCAN,
     NOT,
     NTH_VALUE,
     NTILE,
@@ -305,13 +322,17 @@ define_keywords!(
     ORDER,
     OUT,
     OUTER,
+    OUTPUTFORMAT,
     OVER,
     OVERFLOW,
     OVERLAPS,
     OVERLAY,
+    OVERWRITE,
     PARAMETER,
     PARQUET,
     PARTITION,
+    PARTITIONED,
+    PARTITIONS,
     PERCENT,
     PERCENTILE_CONT,
     PERCENTILE_DISC,
@@ -327,6 +348,7 @@ define_keywords!(
     PREPARE,
     PRIMARY,
     PROCEDURE,
+    PURGE,
     RANGE,
     RANK,
     RCFILE,
@@ -349,6 +371,7 @@ define_keywords!(
     REGR_SYY,
     RELEASE,
     RENAME,
+    REPAIR,
     REPEATABLE,
     REPLACE,
     RESTRICT,
@@ -372,6 +395,7 @@ define_keywords!(
     SELECT,
     SENSITIVE,
     SEQUENCEFILE,
+    SERDE,
     SERIALIZABLE,
     SESSION,
     SESSION_USER,
@@ -380,6 +404,7 @@ define_keywords!(
     SIMILAR,
     SMALLINT,
     SOME,
+    SORT,
     SPECIFIC,
     SPECIFICTYPE,
     SQL,
@@ -389,21 +414,27 @@ define_keywords!(
     SQRT,
     START,
     STATIC,
+    STATISTICS,
     STDDEV_POP,
     STDDEV_SAMP,
     STDIN,
     STORED,
+    STRING,
     SUBMULTISET,
     SUBSTRING,
     SUBSTRING_REGEX,
     SUCCEEDS,
     SUM,
     SYMMETRIC,
+    SYNC,
     SYSTEM,
     SYSTEM_TIME,
     SYSTEM_USER,
     TABLE,
     TABLESAMPLE,
+    TBLPROPERTIES,
+    TEMP,
+    TEMPORARY,
     TEXT,
     TEXTFILE,
     THEN,
@@ -473,9 +504,12 @@ pub const RESERVED_FOR_TABLE_ALIAS: &[Keyword] = &[
     Keyword::SELECT,
     Keyword::WHERE,
     Keyword::GROUP,
+    Keyword::SORT,
     Keyword::HAVING,
     Keyword::ORDER,
     Keyword::TOP,
+    Keyword::LATERAL,
+    Keyword::VIEW,
     Keyword::LIMIT,
     Keyword::OFFSET,
     Keyword::FETCH,
@@ -492,6 +526,8 @@ pub const RESERVED_FOR_TABLE_ALIAS: &[Keyword] = &[
     Keyword::RIGHT,
     Keyword::NATURAL,
     Keyword::USING,
+    Keyword::CLUSTER,
+    Keyword::DISTRIBUTE,
     // for MSSQL-specific OUTER APPLY (seems reserved in most dialects)
     Keyword::OUTER,
 ];
@@ -506,15 +542,20 @@ pub const RESERVED_FOR_COLUMN_ALIAS: &[Keyword] = &[
     Keyword::SELECT,
     Keyword::WHERE,
     Keyword::GROUP,
+    Keyword::SORT,
     Keyword::HAVING,
     Keyword::ORDER,
     Keyword::TOP,
+    Keyword::LATERAL,
+    Keyword::VIEW,
     Keyword::LIMIT,
     Keyword::OFFSET,
     Keyword::FETCH,
     Keyword::UNION,
     Keyword::EXCEPT,
     Keyword::INTERSECT,
+    Keyword::CLUSTER,
+    Keyword::DISTRIBUTE,
     // Reserved only as a column alias in the `SELECT` clause
     Keyword::FROM,
 ];
